/*!\file
 * \brief    ,         I2C.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef I2C_H
#define I2C_H

#include "types.h"
#include "chip_5023BC016.h"
#include "system.h"
#include "gpio.h"  


/**
 * @brief      I2C.
 */

/**
 * @brief       I2C-
 */
typedef struct
{
    __RW uint32_t Clock_prescale;                                               /**<: 0x00   */
    __RW uint32_t Control;                                                      /**<: 0x04   */
    __RW uint32_t Tx_Rx;                                                        /**<: 0x08      */
    __RW uint32_t Com_Stat;                                                     /**<: 0x0C     */
}I2C_T;


/**
 * @brief      Clock_prescale
 */
#define Prescale(x)      ((uint32_t)((x&0xFFFF) << 0))                          /**<         SCL. 
                                                                                           EN   Control   0. 
                                                                                          3*/

/**
 * @brief      Control
 */
#define IEN(x)           ((uint32_t)(x << 6))                                   /**<1   ,    */
#define EN(x)            ((uint32_t)(x << 7))                                   /**<1    I2C    */


/**
 * @brief      Tx_Rx
 */
#define RW(x)            ((uint32_t)(x << 0))                                   /**<     . 1     , 0     */    
#define TDATA(x)         ((uint32_t)((x&0x7F) << 1))                            /**<  .  7   ,      I2C*/
#define RDATA(x)         ((uint32_t)((x&0xFF) << 0))                            /**< ,    I2C*/                    

/**
 * @brief      Com_Stat
 */
#define IACK_IF(x)       ((uint32_t)(x << 0))                                   /**<   (IF)    Com_Stat
                                                                                     ,   ,    .*/         

#define TIP(x)          ((uint32_t)(x << 1))                                    /**<  1,      0    .*/

#define ACK(x)          ((uint32_t)(x << 3))                                    /**<,    . 0   ACK, 1   NACK*/         

#define WR(x)           ((uint32_t)(x << 4))                                    /**<   */         

#define RD_AL(x)        ((uint32_t)(x << 5))                                    /**<   .
                                                                                      1,    */  

#define STO_BUSY(x)     ((uint32_t)(x << 6))                                    /**<  STOP  .
                                                                                       1       START 
                                                                                       0       STOP*/  

#define STA_RxACK(x)    ((uint32_t)(x << 7))                                    /**<  START  ,    START.
                                                                                        (acknowledge). 
                                                                                    0     , 1     */  

#define IF(x)           ((uint32_t)(x << 0))                                    /**< ,   ,    . 
                                                                                       IEN   Control,      . */      

/**
 * @brief       I2C
 */
#define I2C_ENABLE 0x80


/**
 * @brief        / I2C.
 * @param pI2C:      I2C.
 * @retval CtrlI2C:  .
 */
CTRL_T I2C_GetCtrlNum(I2C_T *pI2C);

/**
 * @brief  I2C.
 * @param pI2C:      I2C.
 * @param clt_prsc:   I2C.
 * @retval None.
 */
void I2C_Init(I2C_T *pI2C, uint32_t clt_prsc);

/**
 * @brief         I2C.
 * @param pI2C: None.
 * @retval None.
 */
void I2C_GPIO_Init();

/**
 * @brief     I2C.
 * @param addr:   I2C.
 * @param data:   .
 * @retval Zero or error.
 */
unsigned int I2C_Write_Byte(unsigned char addr, unsigned char data);

/**
 * @brief     I2C.
 * @param addr:   I2C.
 * @param data:  .
 * @retval Zero or error.
 */
unsigned int I2C_Read_Byte(unsigned char addr, unsigned char *data);




#endif /* I2C_H */